/*
* Copyright (C) Mellanox Technologies Ltd, 2001-2018. ALL RIGHTS RESERVED.
*
* See file LICENSE for terms.
*/

#ifndef NVME_MLX5_SIG_H
#define NVME_MLX5_SIG_H

#include <stdint.h>

#define MLX5_BSF_INL_VALID      (1 << 15)
#define MLX5_BSF_REFRESH_DIF    (1 << 14)
#define MLX5_BSF_REPEAT_BLOCK   (1 << 7)

enum mlx5_sig_check_mask {
	MLX5_SIG_CHECK_GUARD  = 0xc0,
	MLX5_SIG_CHECK_APPTAG = 0x30,
	MLX5_SIG_CHECK_REFTAG = 0x0f,
};

enum mlx5_sig_type {
	MLX5_DIF_CRC  = 0x1,
	MLX5_DIF_IPCS = 0x2,
	MLX5_CRC32    = 0x3,
};

struct mlx5_bsf_inl {
	__be16		vld_refresh;
	__be16		dif_apptag;
	__be32		dif_reftag;
	u8		sig_type;
	u8		rp_inv_seed;
	u8		rsvd[3];
	u8		dif_inc_ref_guard_check;
	__be16		dif_app_bitmask_check;
};

struct mlx5_bsf {
	struct mlx5_bsf_basic {
		u8		bsf_size_sbs;
		u8		check_byte_mask;
		union {
			u8	copy_byte_mask;
			u8	bs_selector;
			u8	rsvd_wflags;
		} wire;
		union {
			u8	bs_selector;
			u8	rsvd_mflags;
		} mem;
		__be32		raw_data_size;
		__be32		w_bfs_psv;
		__be32		m_bfs_psv;
	} basic;
	struct mlx5_bsf_ext {
		__be32		t_init_gen_pro_size;
		__be32		rsvd_epi_size;
		__be32		w_tfs_psv;
		__be32		m_tfs_psv;
	} ext;
	struct mlx5_bsf_inl	w_inl;
	struct mlx5_bsf_inl	m_inl;
};


#endif /* NVME_MLX5_SIG_H */
