# Mellanox Device emulation SDK and NVME controller emulation

The SmartNIC comes preinstalled with the source code, compiled
binaries, device emulation library and its .h file.

Source tarball is `/opt/mellanox/nvmx-0.1.tar.gz`
Binaries can be found under `/opt/mellanox/nvmx/`

## Build and run

To build from the source:
- Make sure that prerequisites are installed:
    - custom linux kernel that has devx support
    - custom rdma-core 
- compile sources:
    
    ./autogen.sh && ./configure && make 

To install:

    make install

To run nvme controller emulation code:

    nvme_start mlx5_2

Remove nvme driver on host:

    modprobe -r nvme

    nvme_controller_emu mlx5_2

Load nvme driver on host: 

    modprobe nvme

You will see the following output from `nvme_controller_emu`:

    CQ    [0x30..0x38] : 0x469974000  [Admin Completion Queue Base Address]
    ASQ    [0x28..0x30] : 0x46a9a5000  [Admin Submissiong Queue Base Address]
    AQA    [0x24..0x28] : 0x1f001f ASQS:31 ACQS:31 [Admin Queue Attributes]
    CC     [0x14..0x18] : 0x460001 EN:1 CSS:0 MPS:0 AMS:0 SHN:0 IOSQES:6 IOCQES:4 [Controller Configuration]
    devx_verbs.c:866 WARN devx_ctx 0x1b8e3140 created regular qp 2803 tx_cq 3641 rx_cq 3642
    devx_verbs.c:866 WARN devx_ctx 0x1b8e3140 created on_behalf qp 1152 tx_cq 3641 rx_cq 3642
    devx_verbs.c:866 WARN devx_ctx 0x1b8e3140 created regular qp 2804 tx_cq 3641 rx_cq 3642
    devx_verbs.c:866 WARN devx_ctx 0x1b8e3140 created on_behalf qp 1153 tx_cq 3641 rx_cq 3642
    ACMD: 0x6 IDENTIFY tag: 2 id_ctrl prp1:0x5080e000 prp2:0x0
    ACMD: 0x9 SET_FTRS tag: 2  num_queues: 0x7 sq:23 cq:23
    ACMD: 0x5 CREAT_CQ tag: 2  base:0x46bf8c000 id:1 size:1023 int_v:0x1 flags:0x3
    devx_verbs.c:866 WARN devx_ctx 0x1b8e3140 created regular qp 2805 tx_cq 3641 rx_cq 3642
    devx_verbs.c:866 WARN devx_ctx 0x1b8e3140 created on_behalf qp 1154 tx_cq 3641 rx_cq 3642
    ACMD: 0x1 CREAT_SQ tag: 26  base:0x453930000 id:1 cqid:1 size:1023 flags:0x1
    devx_verbs.c:866 WARN devx_ctx 0x1b8e3140 created regular qp 2806 tx_cq 3641 rx_cq 3642
    devx_verbs.c:866 WARN devx_ctx 0x1b8e3140 created on_behalf qp 1155 tx_cq 3641 rx_cq 3642
    ACMD: 0xc ASYNC_EV_REQ tag: 31  IGNORED
    ACMD: 0x6 IDENTIFY tag: 26 id_ctrl prp1:0x6dba8000 prp2:0x0
    ACMD: 0x6 IDENTIFY tag: 26 nslist prp1:0x6da22000 prp2:0x0
    ACMD: 0x6 IDENTIFY tag: 26 id_ns 1 prp1:0x4ba4e000 prp2:0x0
    ACMD: 0x6 IDENTIFY tag: 26 id_ns 1 prp1:0x4ba4e000 prp2:0x0

After the linux driver is up you will see nvme devices: `/dev/nvme0` and `/dev/nvme0n1`

Note that you only need to run `nvme_start` once after SmartNIC boots.

By default controller code allocates 512MB memory file in `/tmp/nvme_disk.dat`
and presents it as a disk to the host.

The controller initial settings you can find in the src/json/mlnx_nvme_snap_config.json config file.
See comments for possible values and meaning of each field.

## Setting up RDMA target and connecting to it

- configure NVMf/RDMA target (run on remote server `./scripts/config_nvmf_target.py -a <ip_addr> -p <port>`)
- On the Smart Nic configure enp3s0f2 iface: `ifconfig enp3s0f2 3.3.3.1/24 up`. Or add a permanent configuration to the /etc/sysconfig/network-scripts
- check connectivity to the target: `ping 3.3.3.10`
- run nvme_controller_emu:
    
    `env NVME_EMU_TYPE="cc_remote" NVME_EP_TYPE="nvmf_rdma" NVME_EP_ADDR="3.3.3.10" NVME_EP_PORT="4420" NVME_EP_NAME="testsubsystem" samples/nvme_controller_emu mlx5_2`

## Tests

To run tests:
 - `tests/gtest_nvme_ctrl`
 - `tests/gtest_devx` (advanced)

Tests in the `gtest_nvme_ctrl` are to check NVME spec compliance and they can be run on any 
system.

Tests in `gtest_devx` is work in progress, at the moment failures are expected.

## Documentation

To generate documentation:
* Run `doxygen` at the top level project directory
* HTML documentation is generated under `html/`, point your browser to `index.html`

